{-# Language DataKinds #-}
{- |

The scales and guage marks found on a Concise #300 circular slide rule.

-}
module Scales (
  cScale,
  ciScale,
  dScale,
  kScale,
  t1Scale,
  t2Scale,
  sScale,
  ll2Scale,
  ll3Scale,
  lScale,
) where

import SlideRule

lScale :: Scale OnWheel
lScale = Scale
  { scaleName = "L"
  , scaleFun = id
  , scaleFun' = id
  }

cScale :: Scale OnWheel
cScale = Scale
  { scaleName = "C"
  , scaleFun  = logBase 10
  , scaleFun' = (10**)
  }

ciScale :: Scale OnWheel
ciScale = invertScale "CI" cScale

dScale :: Scale OffWheel
dScale = Scale
  { scaleName = "D"
  , scaleFun  = logBase 10
  , scaleFun' = (10**)
  }

aScale :: Scale OffWheel
aScale = Scale
  { scaleName = "A"
  , scaleFun  = logBase 100
  , scaleFun' = (100**)
  }

bScale :: Scale OnWheel
bScale = Scale
  { scaleName = "B"
  , scaleFun  = logBase 100
  , scaleFun' = (100**)
  }

kScale :: Scale OffWheel
kScale = Scale
  { scaleName = "K"
  , scaleFun  = logBase 1000
  , scaleFun' = (1000**)
  }

llScale :: Int -> Scale pos
llScale n  = Scale
  { scaleName = "LL" ++ show n
  , scaleFun  = subtract off . logBase 10 . log
  , scaleFun' = exp . (10**) . (+off)
  }
  where
  off = fromIntegral n - 3

ll3Scale :: Scale OffWheel
ll3Scale = llScale 3

ll2Scale :: Scale OffWheel
ll2Scale = llScale 2

sScale :: Scale OnWheel
sScale = Scale
  { scaleName = "S"
  , scaleFun  = (+1) . logBase 10 . sin . degToRad
  , scaleFun' = radToDeg . asin . (10**) . subtract 1
  }

stScale :: Scale OnWheel
stScale = Scale
  { scaleName = "ST"
  , scaleFun  = (+2) . logBase 10 . sin . degToRad
  , scaleFun' = radToDeg . asin . (10**) . subtract 2
  }

t1Scale :: Scale OnWheel
t1Scale = Scale
  { scaleName = "T1"
  , scaleFun  = (+1) . logBase 10 . tan . degToRad
  , scaleFun' = radToDeg . atan . (10**) . subtract 1
  }

t2Scale :: Scale OnWheel
t2Scale = Scale
  { scaleName = "T2"
  , scaleFun  = logBase 10 . tan . degToRad
  , scaleFun' = radToDeg . atan . (10**)
  }

degToRad :: Double -> Double
degToRad x = x * pi / 180

radToDeg :: Double -> Double
radToDeg x = x * 180 / pi

-- | pi
π :: Double
π = pi

-- | degrees per radian
ρ :: Double
ρ = 18/pi

-- | minutes per radian
ρ' :: Double
ρ' = 10.8/pi

-- | seconds per radian
ρ'' :: Double
ρ'' = 6.48/pi